<?php

namespace EventPortal\Domain\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\ValueObjects\SectionData;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;

class UpdateSections
{
    public function execute(Collection $sections, FormStep $formStep): void
    {
        self::deleteUnusedSections($sections, $formStep);
        $sections->each(static function (SectionData $section) use ($formStep): void {
            $formStepSection = FormStepSection::updateOrCreate(
                ['id' => $section->id],
                Arr::add($section->toArray(), 'form_step_id', $formStep->id)
            );

            (new UpdateQuestions)->execute($section->questions, $formStepSection);
        });
    }

    private static function deleteUnusedSections(Collection $sections, FormStep $formStep): void
    {
        FormStepSection::query()
            ->whereNotIn('id', $sections->pluck('id'))
            ->forFormStep($formStep)
            ->delete();
    }
}
