<?php

namespace EventPortal\Domain\Util\Menu;

use Illuminate\Contracts\Auth\Authenticatable;
use Illuminate\Database\Eloquent\Model;

class Menu
{
    /** @var array */
    protected $context;

    /** @var Authenticatable */
    protected $user;

    /** @var array */
    public $links;

    /** @var array */
    public $buttons;

    /** @var string */
    public $logo;

    public function user(Authenticatable $user): self
    {
        $this->user = $user;

        return $this;
    }

    public function context(Model ...$models): self
    {
        $this->context = $models;

        return $this;
    }

    public function logo(string $logo): self
    {
        $this->logo = $logo;

        return $this;
    }

    public function addLinks(callable $callable): self
    {
        $this->links[] = $callable;

        return $this;
    }

    public function addButtons(callable $callable): self
    {
        $this->buttons[] = $callable;

        return $this;
    }

    public function get(): array
    {
        $this->resolve()
            ->filterPermittedLinks()
            ->render();

        return [
            'logo' => $this->logo,
            'links' => $this->links,
            'buttons' => $this->buttons,
        ];
    }

    private function resolve(): self
    {
        $this->links = collect($this->links)
            ->transform(function ($link) {
                return app()->call($link, $this->getContext());
            })
            ->flatten(1);

        $this->buttons = collect($this->buttons)
            ->transform(function ($button) {
                if ($this->context === null) {
                    return app()->call($button);
                }

                return app()->call($button, $this->getContext());
            })
            ->flatten(1);

        return $this;
    }

    private function filterPermittedLinks(): self
    {
        $this->links = collect($this->links)
            ->filter(function (Link $link) {
                return $link->canSee;
            });

        return $this;
    }

    private function render(): self
    {
        $this->links = collect($this->links)
            ->map(function (Link $link) {
                return $link->render();
            })
            ->values()
            ->toArray();

        $this->buttons = collect($this->buttons)->toArray();

        return $this;
    }

    private function getContext(): array
    {
        return collect($this->context)
            ->mapWithKeys(function (Model $model): array {
                return [$model::KEY => $model];
            })
            ->put('user', $this->user)
            ->all();
    }
}
