<?php

namespace EventPortal\Tests\Feature\Admin\Event;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\EventVenue\Actions\AssociateToEvent;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class EventVenueTest extends TestCase
{
    use RefreshDatabase, SeedRolesAndPermissions;

    /** @test */
    public function can_assign_venue()
    {
        $event = factory(Event::class)->create();
        $venue = factory(Venue::class)->state('has-address')->create();

        $this->actingAsSuperAdmin()
            ->post(route('admin.event.venue.store', $event), [
                $venue->id,
            ]);

        $this->assertContainsModel($venue, $event->fresh()->venues);
        $this->assertCount(1, $venue->fresh()->events, '$venue does not have the correct number of events.');
    }

    /** @test */
    public function can_unassign_venue()
    {
        $assoc = app(AssociateToEvent::class);
        $venue = factory(Venue::class)->state('has-address')->create();
        $events = factory(Event::class, 3)->create()->each(function (Event $event) use ($assoc, $venue) {
            $assoc->execute($event, [$venue->id]);
        });

        $this->assertCount(3, $venue->events, '$venue has the wrong number of events associated to it?');

        $this->assertContainsModel($venue, $events->first()->fresh()->venues);

        $this->actingAsSuperAdmin()
            ->post(route('admin.event.venue.destroy', $events->first()->id), [
                $venue->id,
            ]);

        $this->assertEmpty($events->first()->fresh()->venues, '$events->first still associated?');
        $this->assertCount(2, $venue->fresh()->events, '$venue is still related to event');
    }
}
